package com.caimei365.user.service.impl;

import com.caimei365.user.mapper.*;
import com.caimei365.user.model.po.UserBeansHistoryPo;
import com.caimei365.user.model.vo.UserVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.Date;

/**
 * 异步方法 @Async
 * 类中需要使用@Autowired或@Resource等注解自动注入，不能自己手动new对象
 * 在启动类中增加@EnableAsync注解
 * 在Async 方法上标注@Transactional是没用的。 在Async 方法调用的方法上标注@Transactional 有效。
 * 调用被@Async标记的方法的调用者不能和被调用的方法在同一类中不然不会起作用！！！！！！！
 * 使用@Async时要求是不能有返回值
 * @author : Charles
 * @date : 2021/11/12
 */
@Slf4j
@Component
public class AsyncService {
    @Resource
    private LoginMapper loginMapper;
    @Resource
    private RegisterMapper registerMapper;
    @Resource
    private ClubMapper clubMapper;
    @Resource
    private BaseMapper baseMapper;

    @Async("taskExecutor")
    public void loginUpdateBeans(Integer userId, Integer identity) {
        boolean isClub = null != identity && (2 == identity || 4 == identity);
        if (isClub){
            // 查询今天是否有登录送豆
            Integer id = clubMapper.findLoginBeans(userId);
            if (id == null) {
                //登录赠送10采美豆
                UserVo user = baseMapper.getUserByUserId(userId);
                UserBeansHistoryPo beansHistory = new UserBeansHistoryPo();
                beansHistory.setUserId(user.getUserId());
                beansHistory.setBeansType(12);
                beansHistory.setType(1);
                beansHistory.setNum(10);
                beansHistory.setPushStatus(0);
                beansHistory.setAddTime(new Date());
                registerMapper.insertBeansHistory(beansHistory);
                Integer userBeans = user.getUserBeans();
                userBeans = userBeans == null ? 10 : userBeans + 10;
                clubMapper.updateUserBeans(userId, userBeans);
                log.info("【登录】-----每天初次登录赠送10采美豆，userId:" + userId);
            }
        }
        // 改user表登录时间
        try {
            loginMapper.updateLogin(userId);
            log.info("【登录】-----登陆时间录入，userId:" + userId);
        } catch (Exception e) {
            log.error("【登录】--登录时间记录异常" + e);
        }
    }
}
